const crypto = require('crypto')
const fetch = require('node-fetch')
const qs = require('querystring')

async function fetchPosts ({ url, params }) {
  try {
    const { posts } = await fetch(`${url}/ghost/api/v0.1/posts/?${params}`)
      .then((r) => r.json())

    return posts
  } catch (err) {
    console.error(err)
    return []
  }
}

exports.sourceNodes = async ({ actions, createNodeId }, configOptions) => {
  const { createNode } = actions
  const { clientId = 'ghost-frontend', clientSecret, url } = configOptions

  const params = qs.stringify({
    client_id: clientId,
    client_secret: clientSecret,
    limit: 'all',
    include: 'authors,tags'
  })

  const posts = await fetchPosts({ url, params })

  const nodes = posts.map((post) => {
    const content = JSON.stringify(post)
    const contentDigest = crypto.createHash('md5').update(content).digest('hex')

    return {
      ...post,
      id: `ghost-${post.id}`,
      parent: null,
      children: [],
      internal: {
        type: 'GhostBlogPost',
        content,
        contentDigest
      }
    }
  })

  nodes.forEach((node) => createNode(node))
}

exports.createPages = async ({ graphql, actions }, { posts: { component, path = 'blog' } }) => {
  if (!component) {
    return
  }

  const { createPage } = actions
  const { data: { allGhostBlogPost: { edges: posts } } } = await graphql(`
    {
      allGhostBlogPost {
        edges {
          node {
            title
            html
            slug
          }
        }
      }
    }
  `)

  posts.forEach(({ node: post }) => {
    createPage({
      path: `${path}/${post.slug}/`,
      component: component,
      context: {
        currentPost: post.slug
      }
    })
  })
}
